@@ -1,20 +1,25 @@
 #include "gallery.h"
 #include <QFileSystemWatcher>
+#include <QDateTime>
 
 #include <MauiKit/FileBrowsing/fileloader.h>
 #include <MauiKit/FileBrowsing/fmstatic.h>
 #include <MauiKit/FileBrowsing/tagging.h>
+#include <MauiKit/ImageTools/exiv2extractor.h>
 
 static FMH::MODEL picInfo(const QUrl &url)
 {
     const QFileInfo info(url.toLocalFile());
+    const Exiv2Extractor exiv2(url);
+
     return FMH::MODEL{{FMH::MODEL_KEY::URL, url.toString()},
-                      {FMH::MODEL_KEY::TITLE, info.baseName()},
-                      {FMH::MODEL_KEY::SIZE, QString::number(info.size())},
-                      {FMH::MODEL_KEY::SOURCE, FMStatic::fileDir(url)},
-                      {FMH::MODEL_KEY::DATE, info.birthTime().toString(Qt::TextDate)},
-                      {FMH::MODEL_KEY::MODIFIED, info.lastModified().toString(Qt::TextDate)},
-                      {FMH::MODEL_KEY::FORMAT, info.suffix()}};
+        {FMH::MODEL_KEY::TITLE, info.baseName()},
+        {FMH::MODEL_KEY::SIZE, QString::number(info.size())},
+        {FMH::MODEL_KEY::SOURCE, FMStatic::fileDir(url).toString ()},
+        {FMH::MODEL_KEY::DATE, info.birthTime().toString(Qt::TextDate)},
+        {FMH::MODEL_KEY::MODIFIED, info.lastModified().toString(Qt::TextDate)},
+        {FMH::MODEL_KEY::CITY, exiv2.cityId()},
+        {FMH::MODEL_KEY::FORMAT, info.suffix()}};
 }
 
 Gallery::Gallery(QObject *parent)
@@ -26,11 +31,15 @@ Gallery::Gallery(QObject *parent)
 {
     qDebug() << "CREATING GALLERY LIST";
     m_fileLoader->informer = &picInfo;
-    m_fileLoader->setBatchCount(4000);
+    m_fileLoader->setBatchCount(500);
     connect(m_fileLoader, &FMH::FileLoader::finished, [this](FMH::MODEL_LIST items) {
         Q_UNUSED(items)
-        
+
         emit this->filesChanged();
+        emit this->citiesChanged();
+        emit this->foldersChanged();
+
+        this->setStatus(Status::Ready);
     });
 
     connect(m_fileLoader, &FMH::FileLoader::itemsReady, [this](FMH::MODEL_LIST items) {
@@ -42,10 +51,12 @@ Gallery::Gallery(QObject *parent)
         emit preItemsAppended(items.size());
         this->list << items;
         emit postItemAppended();
+        emit this->countChanged();
     });
 
     connect(m_fileLoader, &FMH::FileLoader::itemReady, [this](FMH::MODEL item) {
         this->insertFolder(item[FMH::MODEL_KEY::SOURCE]);
+        this->insertCity(item[FMH::MODEL_KEY::CITY]);
     });
 
     connect(m_watcher, &QFileSystemWatcher::directoryChanged, [this](QString dir) {
@@ -72,7 +83,6 @@ void Gallery::setUrls(const QList<QUrl> &urls)
     //		return;
 
     this->m_urls = urls;
-    this->clear();
     emit this->urlsChanged();
 }
 
@@ -117,12 +127,21 @@ QStringList Gallery::files() const
 
 void Gallery::scan(const QList<QUrl> &urls, const bool &recursive, const int &limit)
 {
+    if(m_urls.isEmpty())
+    {
+        this->setStatus(Status::Error, "No sources found to scan.");
+        return;
+    }
+
+    this->setStatus(Status::Loading);
     this->scanTags(extractTags(urls), limit);
     m_fileLoader->requestPath(urls, recursive, FMStatic::FILTER_LIST[FMStatic::FILTER_TYPE::IMAGE], QDir::Files, limit);
+
 }
 
 void Gallery::scanTags(const QList<QUrl> &urls, const int &limit)
 {
+    qDebug() << "SCANNING TAGS URLS" << urls;
     FMH::MODEL_LIST res;
     for (const auto &tagUrl : urls) {
         const auto urls = Tagging::getInstance()->getTagUrls(tagUrl.toString().replace("tags:///", ""), {}, true, limit, "image");
@@ -145,8 +164,29 @@ void Gallery::insertFolder(const QUrl &path)
         if (m_autoReload) {
             this->m_watcher->addPath(path.toLocalFile());
         }
+    }
+}
 
-        emit foldersChanged();
+void Gallery::insertCity(const QString & cityId)
+{
+    if (!m_cities.contains(cityId) && !cityId.isEmpty ()) {
+
+        qDebug() << "FOUND CITY <<" << cityId;
+        m_cities << cityId;
+    }
+}
+
+void Gallery::setStatus(const Gallery::Status &status, const QString &error)
+{
+    qDebug() << "Setting up status" << status;
+        this->m_status = status;
+        emit this->statusChanged();
+
+
+    if(error != m_error)
+    {
+        this->m_error = error;
+        emit this->errorChanged(m_error);
     }
 }
 
@@ -162,13 +202,6 @@ QList<QUrl> Gallery::extractTags(const QList<QUrl> &urls)
     });
 }
 
-QVariantMap Gallery::get(const int &index) const
-{
-    if (index >= this->list.size() || index < 0)
-        return QVariantMap();
-    return FMH::toMap(this->list.at(this->mappedIndex(index)));
-}
-
 bool Gallery::remove(const int &index)
 {
     Q_UNUSED(index)
@@ -180,7 +213,7 @@ bool Gallery::deleteAt(const int &index)
     if (index >= this->list.size() || index < 0)
         return false;
 
-    const auto index_ = this->mappedIndex(index);
+    const auto index_ = index;
 
     emit this->preItemRemoved(index_);
     auto item = this->list.takeAt(index_);
@@ -246,13 +279,13 @@ void Gallery::setlimit(int limit)
 int Gallery::indexOfName(const QString &query)
 {
     const auto it = std::find_if(this->items().constBegin(), this->items().constEnd(), [&](const FMH::MODEL &item) -> bool {
-            return item[FMH::MODEL_KEY::TITLE].startsWith(query, Qt::CaseInsensitive);
-        });
+        return item[FMH::MODEL_KEY::TITLE].startsWith(query, Qt::CaseInsensitive);
+    });
 
-        if (it != this->items().constEnd())
-            return this->mappedIndexFromSource(std::distance(this->items().constBegin(), it));
-        else
-            return -1;
+    if (it != this->items().constEnd())
+        return (std::distance(this->items().constBegin(), it));
+    else
+        return -1;
 }
 
 void Gallery::componentComplete()
@@ -260,3 +293,18 @@ void Gallery::componentComplete()
     connect(this, &Gallery::urlsChanged, this, &Gallery::rescan);
     this->reload();
 }
+
+const QStringList &Gallery::cities() const
+{
+    return m_cities;
+}
+
+Gallery::Status Gallery::status() const
+{
+    return m_status;
+}
+
+QString Gallery::error() const
+{
+    return m_error;
+}
