@@ -32,6 +32,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <MauiKit/FileBrowsing/tagging.h>
 #include <MauiKit/FileBrowsing/fmstatic.h>
 
+Pix *Pix::m_instance = nullptr;
+
 Pix::Pix(QObject *parent)
     : QObject(parent)
 {
@@ -94,7 +96,7 @@ QVariantList Pix::sourcesModel() const
     QVariantList res;
     const auto sources = getSourcePaths();
     return std::accumulate(sources.constBegin(), sources.constEnd(), res, [](QVariantList &res, const QString &url) {
-        res << FMStatic::getDirInfo(url);
+        res << FMStatic::getFileInfo(url);
         return res;
     });
 }
